package cmdTemplate

import (
	u "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/client/utils"
	nc "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/nexusclient"
	"github.com/google/uuid"
)

type Create struct {
	Name string
}

func (cmd *Create) GetName() string {
	return cmd.Name
}

func (cmd *Create) GetDesc() []string {
	return []string{
		"Creates a template from an existing VM.",
		"Requires TPL_CREATE user capability."}
}

func (cmd *Create) PrintUsage() {
	for _, desc := range cmd.GetDesc() {
		u.PrintlnErr(desc)
	}
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("USAGE: " + cmd.GetName() + " name access vmID")
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("name      Name of the template to create.")
	u.PrintlnErr("access    Access type, either \"public\" or \"private\".")
	u.PrintlnErr("vmID      ID of the VM used to create the template.")
}

func (cmd *Create) Run(client *nc.NexusClient, args []string) int {
	argc := len(args)
	if argc != 3 {
		cmd.PrintUsage()
		return 1
	}

	name := args[0]
	access := args[1]
	vmID, err := uuid.Parse(args[2])
	if err != nil {
		u.PrintlnErr(err)
	}

	template, err := client.TemplateCreate(vmID, name, access)
	if err != nil {
		u.PrintlnErr(err)
		return 1
	}

	templateStr, err := template.String()
	if err != nil {
		u.PrintlnErr(err)
		return 1
	}
	u.Println("Template successfully created:")
	u.Println(templateStr)
	return 0
}
