package nexusclient

import (
	"encoding/json"

	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/params"
	"github.com/go-resty/resty/v2"
)

func (nc *NexusClient) UserCreate(p params.UserWithPwd) error {
	resp, err := nc.client.R().SetBody(p).Post("/users")
	if err != nil {
		return err
	}
	if resp.IsSuccess() {
		return nil
	} else {
		return errorToMsg(resp)
	}
}

func (nc *NexusClient) UserDelete(email string) error {
	resp, err := nc.client.R().Delete("/users/" + email)
	if err != nil {
		return err
	}
	if resp.IsSuccess() {
		return nil
	} else {
		return errorToMsg(resp)
	}
}

func (nc *NexusClient) UserResetPwd(email string) (string, error) {
	resp, err := nc.client.R().Put("/users/" + email + "/resetpwd")
	if err != nil {
		return "", err
	}

	if resp.IsSuccess() {
		pwd, err := deserializePwd(resp)
		if err != nil {
			return "", err
		}
		return pwd, nil
	} else {
		return "", errorToMsg(resp)
	}
}

func (nc *NexusClient) UserSetCaps(email string, userCaps *params.UserSetCaps) error {
	resp, err := nc.client.R().SetBody(userCaps).Put("/users/" + email + "/caps")
	if err != nil {
		return err
	}
	if resp.IsSuccess() {
		return nil
	} else {
		return errorToMsg(resp)
	}
}

func (nc *NexusClient) UserUnlock(email string) error {
	resp, err := nc.client.R().Put("/users/" + email + "/unlock")
	if err != nil {
		return err
	}
	if resp.IsSuccess() {
		return nil
	} else {
		return errorToMsg(resp)
	}
}

func (nc *NexusClient) UserUpdatePwd(newPwd string) error {
	p := &params.UserSetPwd{Pwd: newPwd}
	resp, err := nc.client.R().SetBody(p).Put("/users/pwd")
	if err != nil {
		return err
	}

	if resp.IsSuccess() {
		return nil
	} else {
		return errorToMsg(resp)
	}
}

// NOTE: this route is not implemented in the backend yet.
func (nc *NexusClient) UserWhoAmI() (*params.UserWithoutPwd, error) {
	resp, err := nc.client.R().Get("/users/whoami")
	if err != nil {
		return nil, err
	}

	if resp.IsSuccess() {
		user, err := deserializeUser(resp)
		if err != nil {
			return nil, err
		}
		return user, nil
	} else {
		return nil, errorToMsg(resp)
	}
}

// NOTE: this route is not implemented in the backend yet.
func (nc *NexusClient) GetUser(email string) (*params.UserWithoutPwd, error) {
	resp, err := nc.client.R().Get("/users/" + email)
	if err != nil {
		return nil, err
	}

	if resp.IsSuccess() {
		user, err := deserializeUser(resp)
		if err != nil {
			return nil, err
		}
		return user, nil
	} else {
		return nil, errorToMsg(resp)
	}
}

func (nc *NexusClient) GetUsers() ([]params.UserWithoutPwd, error) {
	resp, err := nc.client.R().Get("/users")
	if err != nil {
		return nil, err
	}

	if resp.IsSuccess() {
		users, err := deserializeUsers(resp)
		if err != nil {
			return nil, err
		}
		return users, nil
	} else {
		return nil, errorToMsg(resp)
	}
}

// Deserialize a list of users from an http response (no filtering).
func deserializeUsers(resp *resty.Response) ([]params.UserWithoutPwd, error) {
	users := []params.UserWithoutPwd{}
	if err := json.Unmarshal(resp.Body(), &users); err != nil {
		return nil, err
	}
	return users, nil
}

// Deserialize a single user from an http response.
func deserializeUser(resp *resty.Response) (*params.UserWithoutPwd, error) {
	user := params.UserWithoutPwd{}
	if err := json.Unmarshal(resp.Body(), &user); err != nil {
		return nil, err
	}
	return &user, nil
}

// Deserialize user password from http response.
func deserializePwd(resp *resty.Response) (string, error) {
	var pwd *params.UserSetPwd = &params.UserSetPwd{}
	if err := json.Unmarshal(resp.Body(), &pwd); err != nil {
		return "", err
	}
	return pwd.Pwd, nil
}
